def clr():
    print("\033[0m", end="\n")


def color(s, args):
    print(f"\033[{s}", end="")
    for arg in args:
        print(arg, end=" ")
    clr()


def BRED(*args):
    color("1;31m", args)


def BYEL(*args):
    color("1;33m", args)


def BMAG(*args):
    color("1;35m", args)


def BGRE(*args):
    color("1;32m", args)


def BCYA(*args):
    color("1;36m", args)


def BBLU(*args):
    color("1;34m", args)


def RED(*args):
    color("0;31m", args)


def YEL(*args):
    color("0;33m", args)


def MAG(*args):
    color("0;35m", args)


def GRE(*args):
    color("0;32m", args)


def CYA(*args):
    color("0;36m", args)


def BLU(*args):
    color("0;34m", args)



from datetime import date, timedelta


class fastDateRange:
    @staticmethod
    def run(startDate: date, endDate: date, endInclusive=False):
        """
        A quicker way to create a range of dates using the datetime.date datatype than most others I have found. E.g. pandas
        seems o be slow and have a lot of overhead when there is a large volume.

        This will generate a date range including every date from StartDate to EndDate, with the endCapDate being explicitly
        included if endInclusive=True

        The default option of endInclusive=False is present to maintain consistency with

        :param startDate:
        :param endDate:
        :param endInclusive:
        :return:
        """
        for i in range((endDate - startDate).days + endInclusive * 1):
            yield startDate + timedelta(days=i)

    @staticmethod
    def runNDays(startDate: date, nDays: int):
        """
        alternate form of fastDateRange
        :param startDate:
        :param nDays:
        :return:
        """
        endDate = startDate + timedelta(days=nDays)
        return fastDateRange.run(
            startDate=startDate, endDate=endDate, endInclusive=False
        )

    @staticmethod
    def interval(startDate: date, endDate: date, interval: int):
        """
        alternate form of fastDateRange
        :param startDate:
        :param nDays:
        :return:
        """
        total_days = (endDate - startDate).days
        num_intervals = (
            total_days + interval
        ) // interval  # + interval_days to include end date if necessary

        # Generate the date range
        dates = [
            startDate + timedelta(days=interval * i) for i in range(num_intervals + 1)
        ]
        for d in dates:
            yield d



import os


def path_exists(file_path: str) -> None:

    # Extract the directory path
    directory = os.path.dirname(file_path)

    # Check if the directory exists, and if not, create it
    if not os.path.exists(directory):
        os.makedirs(directory)

def has_file_modified(file_path: str, original_mod_time) -> bool:
    current_mod_time = os.path.getmtime(file_path)
    return current_mod_time != original_mod_time

def file_empty(file_path: str) -> bool:
    return os.path.getsize(file_path) == 0

def extensions(file_path: str) -> list[str]:
    abs_path = os.path.abspath(file_path)
    base, ext = os.path.splitext(abs_path)

    # Loop to get all extensions
    extensions = [ext]
    while True:
        base, ext = os.path.splitext(base)
        if ext:
            extensions.append(ext)
        else:
            break

    extensions.reverse()  # To maintain the original order
    
    return extensions


import time
import matplotlib.pyplot as plt

timestamp = str(time.time())

class Benchmark:
    """
    Might create a benchmark tool to track the overheard and performance in each node to identify bottlenecks.
    """

    def __init__(self):
        self.points = {}
        self.categories = {}

    def start(self):
        self.points["start"] = time.time()

    def stop(self):
        self.points["end"] = time.time()

    def mark(self, label: str, category: str = None):
        self.points["label"] = time.time()

    def plot(self):
        pass

    def overall(self):
        pass



from core.util.color import RED, GRE, YEL, BLU, MAG, CYA

# 6.6.24: should write test for this eventually


class Console:

    def _write(self, *args):
        if self.path is not None:
            with open(f"./log/{self.path}", "w") as f:
                f.write(" ".join(map(str, args)))

    def __init__(self, debug=False, mode=0, write=None):
        self.debug = debug
        self.mode = mode
        self.path = write

    def warning(self, *args):
        print("\033[1;35mWARNING: \033[0m", end="")
        for arg in args:
            print(arg, end=" ")
        self._write("WARNING:", *args)

    def error(self, *args):
        print("\033[1;31mERROR: \033[0m", end="")
        for arg in args:
            print(arg, end=" ")
        self._write("ERROR:", *args)

    def red(self, *args, mode=0):
        if self.debug and mode == self.mode:
            RED(args)
        self._write(*args)

    def gre(self, *args, mode=0):
        if self.debug and mode == self.mode:
            GRE(args)
        self._write(*args)

    def yel(self, *args, mode=0):
        if self.debug and mode == self.mode:
            YEL(args)
        self._write(*args)

    def blu(self, *args, mode=0):
        if self.debug and mode == self.mode:
            BLU(args)
        self._write(*args)

    def mag(self, *args, mode=0):
        if self.debug and mode == self.mode:
            MAG(args)
        self._write(*args)

    def cya(self, *args, mode=0):
        if self.debug and mode == self.mode:
            CYA(args)
        self._write(*args)

    def log(self, *args, mode=0):
        if self.debug and mode == self.mode:
            print(*args)
        self._write(*args)
